
set mprint=on.

* This macro computes the mean DVAR within the groups specified by the variable
  list in CVAR.
* The parameters needed to run this macro are:
   INFILE :The datafile containing the information; 
   CVAR  :The list of classification variables;
   DVAR  :The "dependent variable" for which the mean will be computed;
   NJKZ  :The Number of Jackknife replicates;
   JKZ   :The variable with the assignment to JK zones;
   JKR   :The variable with the JK Indicator (0 or 1);
   WGT   :The sampling weight to use.

* Here the macro begins.
define jackgen (infile= !charend('/')   /
                cvar  = !charend('/')   /
                dvar  = !charend('/')   /
                njkz  = !charend('/')   /
                jkz   = !charend('/')   /
                jkr   = !charend('/')   /
                wgt   = !charend('/') ) .

* Turn the weights OFF and select only those not missing DVAR.
get file = !infile.
weight off.
select if not(missing(!dvar)).

* A constant K is added to the file.
compute k = 1.

* The weighted X is computed here.
compute wgtx  = !wgt * !dvar.

* The replicate weights are computed here.
vector rwgt(!njkz).
vector rwgtx(!njkz).
loop #i = 1 to !njkz.
if (!jkz = #i and !jkr = 0)  rwgt(#i) = !wgt * 0.
if (!jkz = #i and !jkr = 1)  rwgt(#i) = !wgt * 2.
if (!jkz <>#i             )  rwgt(#i) = !wgt * 1.

* The replicate weighted X is computed here.
compute rwgtx(#i) = rwgt(#i) * !dvar.

end loop.

* This assigns the name of the general summary file.
!let !nway = tmpjck1

* The main file is summarized here.
aggregate outfile = !nway
  / break = !cvar k
  /       !wgt rwgt1 to !concat(rwgt,!njkz) wgtx rwgtx1 to !concat(rwgtx,!njkz)
    = sum(!wgt rwgt1 to !concat(rwgt,!njkz) wgtx rwgtx1 to !concat(rwgtx,!njkz))
  / n = n(!wgt).

* Rearrange the variable list.
!let !cvar1 = !null
!do !i !in(!cvar)
!let !cvar1 = !concat(!i,!blank(1),!cvar1)
!doend

* Vectors are created here with the classification variables adding K to each.
!let !cvar2a = !concat(!cvar1,!blank(1),k)
!let !cvar2b = !concat(!tail(!cvar1),!blank(1),k)

!do !i !in(!cvar2a)

* BREAK when there are no more variables.
!if (!cvar2b = !null) !then !break
!ifend

get file = !nway.
compute k = 1.

!let !seq = !index(!cvar1,!i)

* Aggregate here the summarized files.
* Once for the specific cells.
aggregate outfile = tmpjckA
  / break = !cvar2a
  /       !wgt rwgt1 to !concat(rwgt,!njkz) wgtx rwgtx1 to !concat(rwgtx,!njkz)
    = sum(!wgt rwgt1 to !concat(rwgt,!njkz) wgtx rwgtx1 to !concat(rwgtx,!njkz))
  / n = sum(n).

*Once for the overall total.
aggregate outfile = tmpjckB
  / break = !cvar2b
  /       twgt  twgt1 to !concat(twgt,!njkz)
    = sum(!wgt rwgt1 to !concat(rwgt,!njkz)).

get file = tmpjckA.
sort cases by !cvar2B.
save outfile = tmpjckA.

* Now match them so that there are cell counts and total counts.
match files
  / file  = tmpjckA
  / table = tmpjckB
  / by !cvar2b.

* This is a temporary variable to identify the levels of classification.
compute type = !seq.

* This will execute if it is the first variable being summarized.
!if (!head(!cvar1)=!head(!cvar2a)) !then
save outfile = jackfnl.
!else
* This will execute for each subsequent file.
add files
  / file = jackfnl
  / file = *.
save outfile = jackfnl.
!ifend.

* Once a variable is summarized, then it is removed from the list.
!let !cvar2a = !tail(!cvar2a).
!let !cvar2b = !tail(!cvar2b).

!doend

* Compute the overall percent in cell and mean.
do if (!wgt > 0).
compute mnx = wgtx / !wgt.
compute pct = 100 * (!wgt / twgt).
end if.

* Initialize the variance variable.
compute mnx_var = 0.
compute pct_var = 0.

* Here do the accumulation for the error term.
vector rwgt = rwgt1  to !concat(rwgt,!njkz).
vector twgt = twgt1  to !concat(twgt,!njkz).
vector wgtx = rwgtx1 to !concat(rwgtx,!njkz).
loop #i = 1 to !njkz.
do if (rwgt(#i)) > 0.
compute mnx_var = mnx_var + (mnx - (wgtx(#i) / rwgt(#i)))**2.
compute pct_var = pct_var + (pct - (100 * (rwgt(#i) / twgt(#i))))**2.
end if.
end loop.

* And take the square root of the variance to get the error term.
compute mnx_se = sqrt(mnx_var).
compute pct_se = sqrt(pct_var).

autorecode type
   / into level.
execute.

select if level=1.

save outfile = final.

!enddefine.

